@@ -5,7 +5,7 @@ $(document).ready(function() {
     // unbind all functions, this ensures that they are never bound more
     // than once.  This is a problem when using jquery ajax tabs
     $('#add_user').unbind();
-    $('.ajax_form .submit').die();
+    $('.object_permissions_form .submit').die();
     $('.user .delete').die();
     $('.group .delete').die();
     $('.permissions').die();
@@ -23,17 +23,20 @@ $(document).ready(function() {
             show: {when:false, ready:true},
             hide: {fixed: true, when:false},
             api:{onShow:function(){
-                $(".ajax_form input[type!=hidden], .ajax_form select").first().focus();
+                $(".object_permissions_form input[type!=hidden], .object_permissions_form select").first().focus();
+                bind_user_perm_form();
             }}
         });
     });
     
     // form submit button
-    $(".ajax_form").live("submit", function(){
-        $("#errors").empty();
-        $(this).ajaxSubmit({success: update});
-        return false;
-    });
+    function bind_user_perm_form() {
+        $(".object_permissions_form").submit(function(){
+            $("#errors").empty();
+            $(this).ajaxSubmit({success: update_user_permissions});
+            return false;
+        });
+    }
     
     // Delete user button
     $('.user .delete').live("click", function() {
@@ -41,10 +44,11 @@ $(document).ready(function() {
         if (confirm("Remove this user: " + name)) {
             $('.qtip').qtip('destroy');
             id = this.parentNode.parentNode.id.substring(5);
-            data = {user:id, permissions:[]};
+            data = {user:id, permissions:[], obj:obj_id};
             $.post(user_url, data,
                 function(code){
-                    if (code==1) {
+                    type = typeof code
+                    if (type=="string") {
                         $("#user_" + id).remove();
                     }
                 },
@@ -57,10 +61,11 @@ $(document).ready(function() {
         name = $(this).parent().parent().children('.name').html();
         if (confirm("Remove this group: "+ name)) {
             id = this.parentNode.parentNode.id.substring(6);
-            data = {group:id, permissions:[]};
+            data = {group:id, permissions:[], obj:obj_id};
             $.post(user_url, data,
                 function(code){
-                    if (code==1) {
+                    type = typeof code
+                    if (type=="string") {
                         $("#group_" + id).remove();
                     }
                 },
@@ -77,24 +82,26 @@ $(document).ready(function() {
                url: this.href,
                title: {text:'Permissions: ', button:'close'}
             },
-            position: {corner:{ target:"topMiddle", tooltip:"bottomMiddle"}},
-            style: {name: 'dark', border:{radius:5}, width:400, background:'#eeeeee', tip: 'bottomMiddle'},
+            position: {corner:{ target:"rightMiddle", tooltip:"leftMiddle"}},
+            style: {name: 'dark', border:{radius:5}, width:400, background:'#eeeeee', tip: 'leftMiddle'},
             show: {when:false, ready:true},
             hide: {fixed: true, when:false},
             api:{onShow:function(){
-                $(".ajax_form input[type!=hidden], .ajax_form select").first().focus();
+                $(".object_permissions_form input[type!=hidden], .object_permissions_form select").first().focus();
+                bind_user_perm_form();
             }}
         });
         return false;
     });
 });
 
-function update(responseText, statusText, xhr, $form) {
+function update_user_permissions(responseText, statusText, xhr, $form) {
     if (xhr.getResponseHeader('Content-Type') == 'application/json') {
-        if (responseText == 1) {
+        type = typeof responseText;
+        if (type == 'string') {
             // 1 code means success but no more permissions
             $('.qtip').qtip('hide');
-            $("#op_users #" + html.attr('id')).remove();
+            $("#op_users #" + responseText).remove();
         } else {
             // parse errors
             errors = responseText
