@@ -15,8 +15,18 @@ urlpatterns = patterns('object_permissions.views.groups',
 )
 
 urlpatterns += patterns('object_permissions.views.permissions',
-    # Users
+    # List all perms for a given user
     url(r'^user/(?P<id>\d+)/permissions/all/?$','all_permissions', name="user-all-permissions"),
+    
+    # add permissions on an object
+    url(r'^user/(?P<user_id>\d+)/permissions/(?P<class_name>\w+)/?$','view_obj_permissions', name="user-add-permissions"),
+    url(r'^group/(?P<group_id>\d+)/permissions/(?P<class_name>\w+)/?$','view_obj_permissions', name="group-add-permissions"),
+    
+    # edit permissions on an object
+    url(r'^user/(?P<user_id>\d+)/permissions/(?P<class_name>\w+)/(?P<obj_id>\d+)/?$', \
+        'view_obj_permissions', name="user-edit-permissions"),
+    url(r'^group/(?P<group_id>\d+)/permissions/(?P<class_name>\w+)/(?P<obj_id>\d+)?$', \
+        'view_obj_permissions', name="group-edit-permissions"),
 )
 
 #The following is used to serve up local media files like images
